#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     entornografico.py
# Purpose:     Entorno grafico para modificacion de iconos.
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        17-Feb-2011.
# Author:    Isabel Aparicio Pérez
# Copyright:    2011 - Isabel Aparicio Pérez    <prog5pe@edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################

import pygtk
import gtk
import gmenu
import sys
from educative_desktop import malla
import os
import pdb
import ConfigParser
import subprocess
import re
from educative_desktop import conversor_icono



class Base:
    
    def crear_ventana(self,titulo,X,Y):
        ##se encarga de crear una ventana con el titulo y el tamaño pasado por parametros
        self.ventana= gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.ventana.set_size_request(X, Y)
        self.ventana.set_title(titulo) 
        self.ventana.show()
        return self.ventana
        
        
    def creaEtiqueta(self,texto,X,Y,alineacion,partido):
        ##se encarga de crear etiquetas con el texto que se le pasa por parametro
        ##el tamaño es x x y
        ##se le pasa la alineacion y por ultimo un valor boleano que indicara si
        ##el texto se parte automaticamente o no
        self.etiqueta=gtk.Label(texto)
        self.etiqueta.set_size_request(X,Y)
        self.etiqueta.set_justify(alineacion)
        self.etiqueta.set_line_wrap(partido)
        self.etiqueta.show()
        return self.etiqueta
        
    def creaBoton(self,nombre,icono):
        ##crea un boton normal con el nombre y el stock pasados por parametros
        self.boton=gtk.Button(nombre,icono)
        self.boton.show()
        return self.boton
        
    def creaBotonBiestado(self,imagen):
        ##crea un boton biestado con la aimagen pasada como parametro
        self.botonbiestado=gtk.ToggleButton()
        self.botonbiestado.add(imagen)
        self.botonbiestado.show()  
        return self.botonbiestado
        
    def creaImagen(self,icono):
        ##crea una imagen
        self.imagen=gtk.Image()
        self.imagen.set_from_file(icono)
        self.imagen.show()
        return self.imagen
        
    def crearTabla(self,x,y,homogeneo):
        ##crea un contenedor tabla
        self.tabla=gtk.Table(x,y,homogeneo)
        self.tabla.show()
        return self.tabla
        
    def crearScroll(self,ajustex,ajustey,estilo,politicax,politicay):
        #crea una ventana de scroll
        self.scroll=gtk.ScrolledWindow(ajustex,ajustey)
        self.scroll.set_shadow_type(estilo)
        self.scroll.set_policy(politicax,politicay)
        self.scroll.show()
        return self.scroll
        
    def crearFrame(self,tipo,nombre,x,y):
        self.frame=gtk.Frame(nombre)
        self.frame.set_shadow_type(tipo)
        self.frame.set_size_request(x,y) ##300,350
        self.frame.show()
        return self.frame
        
    def iconos_rama(self,rama,diccionario):
        ##se encarga de recorrer toda la rama de inicio para buscar todos los programas que estan instalados
        ##los almacena junto con todos los datos en un diccionario de tuplas donde la clave es el nombre
        ##del programa y la tupla el resto de los datos
        for item in rama.contents:
            tupla=[]
            if isinstance (item, gmenu.Directory):
                self.iconos_rama(item,diccionario)
            elif  isinstance (item, gmenu.Entry):
                icono=item.icon
             #   print icono
                if not icono is None:
                    if "." not in item.icon:
                        gtkIcon=self.icon_theme.lookup_icon(item.icon,24,0)
                        if not gtkIcon is None:
                            icono=gtkIcon.get_filename()
                nombre=item.name
                tupla.append(icono)
                tupla.append(item.comment)
                tupla.append(item.exec_info)
                tupla.append(item.launch_in_terminal)
                diccionario[nombre]=tupla
            else:
                pass #ni idea de lo que puede ser
        return diccionario


    def buscaRuta(self,nombreicono):
        ### se encarga de bustar el las rutas siguientes los iconos y si no existe el icono en esa ruta asigna
        ###a ruta el valor ""
        ruta={1:"/usr/share/icons/Faenza/apps/scalable/",2:"/usr/share/icons/Faenza/apps/48/",3:"/usr/share/icons/Faenza/devices/scalable/",\
        4:"/usr/share/icons/hicolor/scalable/apps/",5:"/usr/share/icons/oxygen/256x256/apps/",6:"/usr/share/icons/oxygen/128x128/apps/",\
        7:"/usr/share/icons/hicolor/128x128/apps/",8:"/usr/share/icons/oxygen/64x64/apps/",9:"/usr/share/icons/Faenza/devices/48/",\
        10:"/usr/share/icons/oxygen/48x48/apps/",11:"/usr/share/icons/hicolor/48x48/apps/",12:"/usr/share/pixmaps/"}
        for k,v in ruta.iteritems():
            direc=os.path.join(v,nombreicono)
            if (os.path.exists(direc)):
                ruta=direc
            else:
                ruta="/usr/share/pixmaps/default.png"
        return ruta
    
    
    def creaDialogo(self,nombre):
        ##crea un dialogo informativo que pregunta si esta seguro de eliminar el lanzador.
        texto="   ¿Esta seguro de que desea eliminar el acceso directo del lanzador %s ?" % (nombre)
        dialogo=gtk.MessageDialog(self.ventanaeliminar, gtk.DIALOG_MODAL,gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO,texto)
        dialogo.show() 
        respuesta = dialogo.run()
        dialogo.destroy()
        if respuesta == gtk.RESPONSE_YES:
            dialogo.destroy()
            self.ventanaeliminar.destroy()
        else:
            dialogo.destroy()
            nombre=''
        return nombre 
  
                 
    
    def guardaDatosBoton(self,widget,data):
        dialogo=self.creaDialogo(data[1])
        if dialogo=='':
            pass
        else:
            listaiconos2=[]
            cadena=''           
            fichero=os.path.expanduser("~/.config/aplicaciones.cfg")
            if not os.path.exists(fichero): #then use the general configuration
                try:
                    copy('/etc/aplicaciones.cfg',fichero)
                    copy('/etc/aplicaciones.cfg',"~/.config/aplicaciones.cfg",fichero)
                except:
                   pass #no solution if not pattern            
                       
            cfg=ConfigParser.ConfigParser()
            if not cfg.read(fichero):
                print ("No se ha podido leer el fichero aplicaciones.cfg")
            else:
                valores=cfg.get('categorias',data[0])
                listaiconos=valores.split(',')
                for i in listaiconos:
                    if i==data[1]:
                        pass
                    else:
                        listaiconos2.append(i)
                for i in listaiconos2:
                    if cadena=='':
                        cadena=i
                    else:
                        cadena=cadena+','+i
                cfg.set("categorias",data[0],cadena)
                f=open(fichero,"w")
                cfg.write(f)
                f.close()
                self.refrescar_escritorio()
 
    def aniadirCancelar(self,widget,data):
        ##se encarga de destruir la ventana
        self.ventana.destroy()
    
    def conviertenombre(self,nombre):
        ##cambia los caracteres del nombre pasado como parametro y los reemplaza por caracteres sin
        ##acentos o en caso de caracteres que no sean letras por caracteres vacios
        nombre=nombre.replace('á','a')
        nombre=nombre.replace('é','e')
        nombre=nombre.replace('í','i')
        nombre=nombre.replace('ó','o')
        nombre=nombre.replace('ú','u')
        nombre=nombre.replace('Á','A')
        nombre=nombre.replace('É','E')
        nombre=nombre.replace('Í','I')
        nombre=nombre.replace('Ó','O')
        nombre=nombre.replace('Ú','U')
        nombre=nombre.replace('Ñ','Ñ')
        nombre=nombre.replace('ñ','ñ')  
        nombre=nombre.replace('«','')
        nombre=nombre.replace('»','')
        nombre=nombre.replace('.','')
        nombre=nombre.replace('-','')
        nombre=nombre.replace('_','')        
        nombre=nombre.replace('!','')
        return nombre
        
    def aniadirAceptar(self,widget,data):
        fichero=os.path.expanduser("~/.config/aplicaciones.cfg")
        if not os.path.exists(fichero): #then use the general configuration
            try:
                copy('/etc/aplicaciones.cfg',fichero)
            except:
               pass #no solution if not pattern       
        cfg=ConfigParser.ConfigParser()
        if not cfg.read(fichero):
                print ("No se ha podido leer el fichero aplicaciones.cfg")
        else:
            categorias=data[0]
            nombre= data[1]
            nombreoriginal=data[6]
            nombreoriginal=self.conviertenombre(nombreoriginal)
            nomseccion=self.conviertenombre(nombreoriginal)            
            nomseccion=nomseccion.lower()
            nomseccion=nomseccion.replace(' ','')            
            nombre1 = nombre.get_text()
            nombre1 = nombre1.lower()
            ruta=data[2]         
            
            

            iconocreado=conversor_icono.ConversorIcono(ruta,nombre)
            iconocreado.main()

            ejecutable=data[3]
            print data[3]
            print ejecutable
            terminal=data[4]
            combo=data[5]
            combo1 = combo.entry.get_text()
            combo1=combo1.lower()
            if combo1=="imagen y sonido":
                combo1="imagenysonido"
            valores=cfg.get('categorias',combo1)
            valores=valores+','+nomseccion
            cfg.set('categorias',combo1,valores)
            if cfg.has_section(nomseccion):
                texto="   Se ha añadido la aplicacion %s correctamente" % (nombre1)
                dialogo=gtk.MessageDialog(self.ventanaaniadir, gtk.DIALOG_MODAL,gtk.MESSAGE_INFO, gtk.BUTTONS_OK,texto)
                dialogo.show() 
                respuesta = dialogo.run()
                dialogo.destroy()
                if respuesta == gtk.RESPONSE_OK:
                    dialogo.destroy()
                    self.ventanaaniadir.destroy()
                    self.ventana.destroy()
            else:
                cfg.add_section(nomseccion)
                cfg.set(nomseccion,"iconsii",'')
                cfg.set(nomseccion,"exec",ejecutable)
                cfg.set(nomseccion,"terminal",terminal)
                cfg.set(nomseccion,"name[es_ES]",nombre1)
                cfg.set(nomseccion,"type","Application")
                cfg.set(nomseccion,"icon",ruta)
                texto="   Se ha añadido la aplicacion %s correctamente" % (nombre1)
                dialogo=gtk.MessageDialog(self.ventanaaniadir, gtk.DIALOG_MODAL,gtk.MESSAGE_INFO, gtk.BUTTONS_OK,texto)
                dialogo.show() 
                respuesta = dialogo.run()
                dialogo.destroy()
                if respuesta == gtk.RESPONSE_OK:
                    dialogo.destroy()
                    self.ventanaaniadir.destroy()
                    self.ventana.destroy()

            f = open(fichero, "w")  
            cfg.write(f)  
            f.close()  
            self.refrescar_escritorio()
        
      


    
    def aniadir(self,widget,data):
        nombre=data[1]
        nombreoriginal=data[1]
        categorias=data[0]
        ruta=data[2]
        #ventana principal
        self.ventana=self.crear_ventana("Añadir Lanzador",500,450)
        self.ventana.set_border_width(10)
        tabla=gtk.Table(8,8,True)
        self.ventana.add(tabla)
        #etiqueta principal
        texto="   Va a añadir el icono de %s como lanzador, selecciones las opciones correctas:" % (nombre)
        etiqueta=self.creaEtiqueta(texto,500,50,gtk.JUSTIFY_CENTER,True)
        tabla.attach(etiqueta,0,8,0,1)
        #frame con combo y etiqueta de categoria
        frame=self.crearFrame(gtk.SHADOW_OUT,None,100,100)
        
        #frame=gtk.Frame()
        #frame.set_shadow_type(gtk.SHADOW_OUT)
        vbox1=gtk.VBox(False,0)
        texto1="Elija la categoria donde desea colocar el acceso directo y debajo el nombre, si lo desea puede dejar el que aparece por defecto:"
        etiqueta1=self.creaEtiqueta(texto1,450,50,gtk.JUSTIFY_CENTER,True)
        vbox1.pack_start(etiqueta1,False,False,0)
        combo=gtk.Combo()
        combo.set_popdown_strings(categorias)
        combo.show()
        vbox1.pack_start(combo,True,False,10)
        frame.add(vbox1)
        tabla.attach(frame,0,8,1,4)

        # caja de entrada de texto de nombre
        etiquetaNombre=self.creaEtiqueta("Nombre",70,10,gtk.JUSTIFY_LEFT,False)
        tabla.attach(etiquetaNombre,0,2,3,4)
        nombre=gtk.Entry(0)
        nombre.set_text(data[1])
        nombre.set_editable(True)
        tabla.attach(nombre,2,8,3,4)

        #caja de entrada de texto de ejecutable
        etiquetaEjecutable=self.creaEtiqueta("Ejecutable",70,10,gtk.JUSTIFY_LEFT,False)
        tabla.attach(etiquetaEjecutable,0,2,4,5)
        ejecutable1=gtk.Entry(0)
        ejecutable1.set_text(data[3])
        ejecutable1.set_editable(False)
        ejecutable1.set_sensitive(False)
        tabla.attach(ejecutable1,2,8,4,5)
        #check de terminal
        checkterminal=gtk.CheckButton("Ejecución desde terminal")
        if data[4]==True:
            checkterminal.set_active(True)
        else:
            checkterminal.set_active(False)
        checkterminal.set_sensitive(False)    
        tabla.attach(checkterminal,4,8,5,7)

        #imagen del icono
        icono=gtk.Image()
        icono.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(ruta).scale_simple(48,48,gtk.gdk.INTERP_BILINEAR))
        boton=self.creaBotonBiestado(icono)
        tabla.attach(boton,1,3,5,7)        
        #caja de botones
        cajabotones=gtk.HButtonBox()
        cajabotones.set_layout(gtk.BUTTONBOX_END)
        botonaceptar=self.creaBoton("Aceptar",gtk.STOCK_ADD)
        botoncancelar=self.creaBoton("Cancelar",gtk.STOCK_CANCEL)
        botonaceptar.connect("clicked",self.aniadirAceptar,(categorias,nombre,ruta,ejecutable1,checkterminal,combo,nombreoriginal))
        botoncancelar.connect("clicked",self.aniadirCancelar,None)
        cajabotones.add(botonaceptar)
        cajabotones.add(botoncancelar)
        tabla.attach(cajabotones,0,8,7,8)        
        #visualizacion de controles
        tabla.show()
        cajabotones.show()
        icono.show()
        etiquetaNombre.show()
        checkterminal.show()
        etiquetaEjecutable.show()
        vbox1.show()
        nombre.show()
        ejecutable1.show()
        self.ventana.show()

        
                 
    def aniadir_icono(self,widget,data=None):
        diccionario1={}
        aplicaciones=[]
        aplicaciones2=[]
        diccionario={}
        categorias=["Educacion","Imagen y Sonido","Herramientas","Internet","Juegos"]
        self.ventanaaniadir=self.crear_ventana("Aniadir Iconos",600,600)
        self.ventanaaniadir.set_border_width(30)
        vbox1=gtk.VBox(False,0)
        self.ventanaaniadir.add(vbox1)
        texto="   Elija el icono de la aplicación que desea añadir como acceso directo en su escritorio:"
        etiquetaAniadir=self.creaEtiqueta(texto,600,50,gtk.JUSTIFY_CENTER,True)
        vbox1.pack_start(etiquetaAniadir,False,True,10)
        
        ##ventana con scroll
        sw=self.crearScroll(None,None,gtk.SHADOW_ETCHED_IN,gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        vbox1.pack_start(sw,True,True,10)
        tabla=gtk.Table(3,100,False)
        sw.add_with_viewport(tabla)
        
        ##########################################3
        
        
        self.icon_theme = gtk.icon_theme_get_default()
        arbol=gmenu.lookup_tree("applications.menu", gmenu.FLAGS_INCLUDE_EXCLUDED)
        raiz=arbol.root
        for rama in arbol.root.contents: #ramas del menu
            diccionario=self.iconos_rama(rama,diccionario)
            for opt,value in diccionario.iteritems():
                if value[0]==None:
                    value[0]="/usr/share/pixmaps/default.png"
                diccionario1[opt]=value
        a=0
        b=1
        for opt,value in diccionario1.iteritems():
            if a<4:
                ruta=value[0]
                ejecutable=value[2]
                terminal=value[3]
                if ruta=="None":
                    ruta="/usr/share/pixmaps/default.png"
                if ruta[0:1] != "/" and ruta != "":
                    ruta=self.buscaRuta(value[0])
                cajaVer=gtk.VBox(True,0)
                imagen=gtk.Image()
                if ruta=="":
                    ruta="/usr/share/pixmaps/default.png"
                imagen.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(ruta).scale_simple(48,48,gtk.gdk.INTERP_BILINEAR))
                imagen.show()
                etiqueta=self.creaEtiqueta(opt,90,50,gtk.JUSTIFY_CENTER,True)
                cajaVer.pack_start(imagen,False,False,0)
                cajaVer.pack_start(etiqueta,True,True,0)
                cajaVer.show() 
                botonbiestado=self.creaBotonBiestado(cajaVer)
                botonbiestado.connect("clicked",self.aniadir,(categorias,opt,ruta,ejecutable,terminal))
                tabla.attach(botonbiestado,a,a+1,b,b+1)
                a=a+1
                tabla.show()
            else:
                b=b+1
                a=0
                ruta=value[0]
                ejecutable=value[2]
                terminal=value[3]
                if ruta[0:1] != "/":
                    ruta=self.buscaRuta(value[0])
                cajaVer=gtk.VBox(True,0)
                imagen=gtk.Image()
                if ruta=="":
                    ruta="/usr/share/pixmaps/default.png"
                imagen.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(ruta).scale_simple(48,48,gtk.gdk.INTERP_BILINEAR))
                etiqueta=self.creaEtiqueta(opt,90,50,gtk.JUSTIFY_CENTER,True)
                imagen.show()
                cajaVer.pack_start(imagen,False,False,0)
                cajaVer.pack_start(etiqueta,True,True,0)
                botonbiestado=self.creaBotonBiestado(cajaVer)
                botonbiestado.connect("clicked",self.aniadir,(categorias,opt,ruta,ejecutable,terminal))
                cajaVer.show() 
                tabla.attach(botonbiestado,a,a+1,b,b+1)
                a=a+1
                tabla.show()                
        vbox1.show()
        self.ventanaaniadir.show()
        
    def sacaIcono(self):      
             ########
     ######## CON ESTE BUCLE SACAMOS EN CATNOMICO UN DICCIONARIO QUE CONTIENE
     ##TUPLAS CON EL NOMBRE Y EL ICONO Y ADEMAS ELIMINAMOS CATEGORIAS Y CARPETAS
     ##QUE NO QUEREMOS PINTAR
        catNomIco={}
        lis2=[]
        malla.inicializa()
        iconos=malla.Iconos
        for opt,val in iconos.iteritems():
            if opt=='categorias' or opt=='carpetas':
                continue
            else:
                catNomIco[opt]=[]
                for opt2,val2 in val.iteritems():
                    lis=[]
                    lis.append(opt2)
                    lis.append(val2[4])
                    lis2.append(lis)
                catNomIco[opt]=lis2
                lis2=[]
        return catNomIco
    
    def eliminar_icono(self,widget,data=None):
        cat=[]
        cat2=[]
        catNomIco={}
        lis2=[]
        categorias={"educacion":"Educación","imagenysonido":"Imagen y Sonido","herramientas":"Herramientas","internet":"Internet","juegos":"Juegos"}
        self.ventanaeliminar=self.crear_ventana("Eliminar Iconos",850,550)  ##600,500
        self.ventanaeliminar.set_border_width(10)
        vbox1=gtk.VBox(False,0)
        self.ventanaeliminar.add(vbox1)
        texto="Escoja la aplicacion que desea eliminar dentro de la pestaña correspondiente:"
        etiquetaEliminar=self.creaEtiqueta(texto,850,50,gtk.JUSTIFY_CENTER,True)
        vbox1.pack_start(etiquetaEliminar,False,True,10)   
        catNomIco=self.sacaIcono()
         ### vamos ha hacer esto con un control de fichas
        fichas=gtk.Notebook()
        fichas.set_tab_pos(gtk.POS_TOP)
        #con la orden siguiente nos aseguramos que si en el futuro queremos aniadir mas fichas y no
        #hay espacio suficiente van a aparecer unas flechas para mostrar el resto de las fichas
        fichas.set_scrollable(True)
        fichas.show()
        for opt in categorias:
            cat.append(categorias[opt])
            cat2.append(opt)
        for i in range(len(categorias)):
            nombre=cat[i]
            nombre2=cat2[i]
            frame=self.crearFrame(gtk.SHADOW_OUT,nombre,500,400)
            pestania=gtk.Label(nombre)
            fichas.append_page(frame,pestania)
           ####ahora lo que hay que hacer es con el nombre y la ruta del icono y generar una imagen
        ###y añadirla al frame. 
        
            ###creamos una tabla y la aniadimos al frame
            tabla=self.crearTabla(3,7,True) 
            frame.add(tabla)
            a=0
            b=0
            for opt,value in catNomIco.iteritems():
                if opt==nombre2:  
                    for i in value:
                        cont=0
                        for j in i:
                            if cont==0:
                                nombre=j
                                cont=cont+1
                            else:
                                icono=j
                        if a<=6:              
                            imagen=self.creaImagen(icono)
                            botonbiestado=self.creaBotonBiestado(imagen)
                            tabla.attach(botonbiestado,a,a+1,b,b+1)
                            a=a+1
                            botonbiestado.connect("clicked",self.guardaDatosBoton,(nombre2,nombre,fichas))
                        else:
                            a=0
                            b=b+1
                            imagen=self.creaImagen(icono)
                            botonbiestado=self.creaBotonBiestado(imagen)
                            tabla.attach(botonbiestado,a,a+1,b,b+1)
                            a=a+1
                            botonbiestado.connect("clicked",self.guardaDatosBoton,(nombre2,nombre,fichas))
            vbox1.pack_start(fichas,False,True,10)
            pestania.show()           
          
        vbox1.show()
        self.ventanaeliminar.show()
    
        
    def delete_event (self,widget,data=None):
        gtk.main_quit()
        return False
        
        
        
    def refrescar_escritorio(self):    
        current_pid = subprocess.Popen("pidof -x escritorio_educativo.py",stdout=subprocess.PIPE,shell=True).communicate()[0].strip()
        if current_pid!='':
            try:
                os.kill(int(current_pid),9)
                subprocess.Popen("/usr/bin/escritorio_educativo.py")
            except:
                pass #por lo que sea no pudo matar el escritorio viejo
            
    def destroy (self,widget,data=None):
        gtk.main_quit()
    
    
    def __init__(self):
        #creamos la ventana principal
        malla.inicializa()
        self.ventanappal = self.crear_ventana("Modificación Iconos",600,100)
        self.ventanappal.connect("delete_event",self.delete_event)

        vbox=gtk.VBox(False, 0)
        self.ventanappal.add(vbox)


        texto= "   Esta herramienta sirve para modificar los accesos directos de sus categorias. Elija la opción deseada para manipular los iconos de los lanzadores"
        etiquetaInicial=self.creaEtiqueta(texto,600,150,gtk.JUSTIFY_CENTER,True)      
        vbox.pack_start(etiquetaInicial,True,False,7)
        
        barraBotones=gtk.HButtonBox()
        barraBotones.set_layout(gtk.BUTTONBOX_SPREAD)
        botonaniadir=self.creaBoton("Añadir",gtk.STOCK_ADD)
        botonaniadir.connect("clicked",self.aniadir_icono)
        barraBotones.add(botonaniadir)
        botoneliminar=self.creaBoton("Borrar",gtk.STOCK_DELETE)       
        botoneliminar.connect("clicked",self.eliminar_icono)
        barraBotones.add(botoneliminar)
        botonsalir=self.creaBoton("Salir",gtk.STOCK_QUIT)
        botonsalir.connect("clicked",self.delete_event)
        barraBotones.add(botonsalir)
        
        vbox.pack_start(barraBotones,False,False,15)
        barraBotones.show()
        vbox.show()
        self.ventanappal.show()

   
    def main(self):
        gtk.main()
        
        
if __name__ == "__main__":
    base = Base()
    base.main()
