#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     escritorio_educativo.py
# Purpose:     Módulo de inicio de la aplicación
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        29-Apr-2011.
# Author:    Antonio Moreno
# Copyright:    2011 - Antonio Moreno       <antoniomoreno @nospam@ edu.juntaextremadura.net>
#                                José L. Redrejo Rodríguez       <jredrejo @nospam@ edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################

import os
import educative_desktop.malla as malla
import educative_desktop.representa_icono
import gtk
import subprocess
import sys
from time import sleep
import ConfigParser

class ColocaEscritorioInicial():
    """Abstraccion de objetos que permiten tener en el escritorio
    los iconos de carpetas y de categorías"""
    #######
    # se realiza el arranque del escritorio; lee de aplicaciones.cfg toda la informacion
    # Para representar los iconos requiere:
    # representa_icono.py, malla.py, y comun.py
    #######

    def __init__(self):
        """Módulo de inicio de la creación del escritorio docente"""
        # para disponer de transparencias y sombrados se dan 3 casos:
            # 1.- portátiles docentes y de estudiante: compiz
            # 2.- terminales en las aulas de Secundaria: metacity con composite
            # 3.- equipos Inves en colegios de Primaria: xcompmgr 
        # en cuál de ellos estamos?:
        COMPIZ = subprocess.Popen("ps ax | grep compiz* | grep -v grep | wc -l", 
                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                  shell=True);
        existeCOMPIZ = COMPIZ.stdout.read()
        # si existe compiz no hay más que hacer, caso 1
        if int(existeCOMPIZ) == 0:
            # veamos si existe metacity:
            METACITY = subprocess.Popen("ps ax | grep metacity | grep -v grep | wc -l", 
                                        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                        shell=True);
            existeMETACITY = METACITY.stdout.read()           
            if int(existeMETACITY) == 0:
            # entonces no están cargados ni compiz ni metacity, queda la opción kwin:
                KWIN = subprocess.Popen("ps ax | grep kwin | grep -v grep | wc -l", 
                                         stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                         shell=True);
                existeKWIN = KWIN.stdout.read()
                if int(existeKWIN) == 0:
                    # entonces se necesita xcompmgr para las sombras y transparencias
                    XCOMPMGR = subprocess.Popen("ps ax | grep xcompmgr | grep -v grep | wc -l", 
                                                stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
                                                shell=True);
                    existeXCOMPMGR = XCOMPMGR.stdout.read()
                    if int(existeXCOMPMGR) == 0:
                        try:
                            subprocess.Popen(['xcompmgr'])
                        # otras opciones:
                        # (['xcompmgr', '-fFcC', '-Sn', '&', 'killall', 'gnome-panel', '&'])
                        # ('xcompmgr -CcfF -I-.015 -O-.03 -D6 -t-1 -l-3 -r4.2 -o.5', shell=True)
                        # (['xcompmgr', '-fF', '-I-.003', '-O-.003', '-D2', '-cC', '-t-4', '-l-6', '-r6', '&'])
                        except:
                            print "Error, no existe el programa xcompmgr"
                            sys.exit()                            
                        sleep(2)
            else: #de el if de int(existeMETACITY) == 0:
                 #entonces estamos en los terminales de Secundaria, con metacity: activamos compositing:
                subprocess.Popen("gconftool-2 --type bool --set /apps/metacity/general/compositing_manager false",
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True);

        #usando composite en terminales ligeros hay que decirles que consuman menos recursos:     
        if "LTSP_CLIENT" in os.environ.keys():
            subprocess.Popen("gconftool-2 --type bool --set /apps/metacity/general/reduced_resources true",
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

       
    def categorias(self):
        """Coloca en el escritorio los iconos de las categorías;
        accedemos a la lista de coordenadas y a los iconos correspondientes"""
        N = malla.CATEGORIAS 
        for p in range(N):
            dato=malla.Iconos['categorias'].values()[p]
            icono1 = dato[4] # desactivado
            icono2 = dato[5] # activado
            instruccion = dato[3]
            factor = malla.Factor
            nombre = dato[2]
            coordenadas = malla.Coordenadas[1][p]
            iconos = []
            icono_a_dibujar = icono1
            activo = False
            tipo = 1
            categoria = educative_desktop.representa_icono.RepresentaIcono(factor, icono1, icono2,
                                                        icono_a_dibujar, instruccion, 
                                                        nombre, coordenadas, 
                                                        iconos, activo, tipo)
            malla.IconosCategorias[nombre]=categoria
            categoria.main()
            categoria.set_keep_below = True

		            
    def carpetas(self):
        """Coloca en el escritorio los iconos de las carpetas;
        accedemos a la lista de coordenadas y a los iconos correspondientes.
        Se debe pasar como parámetro la lista de carpetas"""
        N = malla.CARPETAS
        for p in range(N):
            dato= malla.Iconos['carpetas'].values()[p]
            icono1 =dato[4]
            icono2 = dato[5] # cuando está activado
            instruccion = dato[3]
            factor = malla.Factor
            nombre = dato[2]
            coordenadas = malla.Coordenadas[2][p]
            iconos = ''
            icono_a_dibujar=icono1
            activo = False
            tipo = 2
            carpeta = educative_desktop.representa_icono.RepresentaIcono(factor, icono1, icono2,
                                                      icono_a_dibujar, instruccion, 
                                                      nombre, coordenadas, 
                                                      iconos, activo, tipo)
            malla.IconosCarpetas[nombre]=carpeta
            carpeta.main()
            carpeta.set_keep_below = True
            
def iconos_sueltos():
    #lectura de iconos    
    fichero=os.path.expanduser("~/.config/iconos_sueltos.cfg")
    if not os.path.exists(fichero): return
    cfg = ConfigParser.ConfigParser()
    factor=malla.Factor
    cfg.read(fichero)
    for i in cfg.sections():
        opciones={}
        for n in cfg.items(i):
            opciones[n[0]]=n[1]
  
        coordenadas=(int(factor * int(opciones["x"])), int(factor*175 + int(opciones["y"])))
        lanzador=educative_desktop.representa_icono.RepresentaIcono(malla.Factor, '', '',
                                                              opciones["icon"], opciones['exec'], 
                                                              opciones["name[es_es]"], coordenadas, 
                                                              '', True, 3)
        lanzador.main()
        lanzador.set_keep_below = True   

if  __name__ == '__main__':
    estructura = ColocaEscritorioInicial()
    # recoge información generada del fichero de configuración y cálculos: 
    malla.inicializa()
    #dibuja las categorías de arriba y las carpetas:
    estructura.categorias()
    if malla.CARPETASVACIO==False:
        estructura.carpetas()
    #si puede, hace el chequeo, esto solo funcionará en un instituto extremeño:
    try:                
        if os.path.exists("/usr/bin/chequeo"):
            subprocess.Popen("/usr/bin/chequeo", shell=True);
    except:
        pass
    
    
    #llama a los iconos sueltos si los hubiera 
    iconos_sueltos()    
        
        
    #arranca finalmente el bucle de eventos:
    gtk.main()
