#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     escritorio_educativo.py
# Purpose:     Módulo de personalización de las ventanas de los iconos
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        17-Feb-2011.
# Author:    Antonio Moreno
# Copyright:    2011 - Antonio Moreno       <antoniomoreno @nospam@ edu.juntaextremadura.net>
#                                José L. Redrejo Rodríguez       <jredrejo @nospam@ edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################
##
# este módulo es importado por representaIcono.py para que 
# sus gtkWindows con la instruccion DOCK permitan acceder al resto del escritorio
# sin que se oculten; actualmente el código de WINDOW_TYPE_HINT_DOCK 
# definido en representaIcono.py para las ventanas de los iconos no permite
# llevar las ventanas gtkWindows a segundo plano.
# Agradecimientos a RainCT (http://bloc.eurion.net/)
##

import gtk
import cairo

def exponer_transparencia(widget, event):
    cr = widget.window.cairo_create()
    cr.set_operator(cairo.OPERATOR_CLEAR)
    region = gtk.gdk.region_rectangle(event.area)
    cr.region(region)
    cr.fill()
    return False

class DesktopWindow(gtk.Window):
    # Personalizacion de una GtkWindow. 
    # Usaremos desktopWindow en vez de gtkWindow en los iconos
    def __init__(self, *args):
        # será una gtkwindow con los argumentos que la vamos a pasar:
        gtk.Window.__init__(self, *args)
        # no minimizable, en el fondo y sin bordes:
        self.set_skip_taskbar_hint(True)
        self.set_skip_pager_hint(True)
        self.set_keep_below(True)
        self.set_decorated(False)
        self.stick()
        # pintable y transparente:
        screen = self.get_screen()
        rgba = screen.get_rgba_colormap()
        self.set_colormap(rgba)
        self.set_app_paintable(True)
        self.connect("expose-event", exponer_transparencia)
