#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     conversor_icono.py
# Purpose:     Módulo de creación de iconos
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        17-Feb-2011.
# Author:    Antonio Moreno
# Copyright:    2011 - Antonio Moreno       <antoniomoreno @nospam@ edu.juntaextremadura.net>
#                                José L. Redrejo Rodríguez       <jredrejo @nospam@ edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################


import os
import sys
import subprocess
import commands
import shutil
from PIL import Image

global icono_orig

class ConversorIcono(object):
    """Abstraccion de objetos conversores de icono de serie a icono personalizado"""
    def __init__(self, icono_orig, titulo):
        """método que se llama al instanciar la clase: incluye 
        los procesos de inicialización necesarios para crear la instancia"""
        # datos de entrada:
        self.icono_orig = icono_orig
        self.titulo = titulo
        numversionIM = 0.0
    
    ##
    # FUNCIONES:
    ##
    
    def comprueba_im_opti(self):
        """Se requiere ImageMagick y Optipng"""
        IM = subprocess.Popen("which convert", stdout=subprocess.PIPE, 
                              stderr=subprocess.PIPE, shell=True);
        existeIM = IM.stdout.read()
        if existeIM == "":
            print "Error, no existe el programa ImageMagick"
            sys.exit()
        versionIM = subprocess.Popen("convert -version", stdout=subprocess.PIPE, 
                                     stderr=subprocess.PIPE, shell=True);
        global numversionIM
        self.numversionIM = float(versionIM.stdout.read()[21:24])
        print "Se encuentra ImageMagick "+ str(self.numversionIM)
        Opti = subprocess.Popen("which optipng", stdout=subprocess.PIPE, 
                                stderr=subprocess.PIPE, shell=True);
        existeOpti = Opti.stdout.read()
        if existeOpti == "":
            print "Error, no existe el programa Optipng"
            sys.exit()    
    
    def asegura_png(self):
        """El icono debe estar en PNG y en RGB desde el principio.
        El método trabaja sobre el icono original"""
        # si el icono es GIF hay que hacer los blancos transparentes:
        if self.icono_orig[-3:] == 'gif' or self.icono_orig[-3:] == 'GIF':
            conv = subprocess.Popen(['mogrify', '-strip', '-set', 'colorspace', 'sRGB', '-type', 'TrueColorMatte', '-background', 'transparent', '-format', 'png', '-transparent', 'white', '-level', '12%,100%,0.9', '-enhance', str(self.icono_orig)])
        # para cualquier otro formato:
        else:
            conv = subprocess.Popen(['mogrify', '-strip', '-type', 'TrueColorMatte', '-background', 'transparent', '-set', 'colorspace', 'sRGB', '-format', 'png', '-modulate', '60,110,100', '-sigmoidal-contrast', '5', '-contrast', '-contrast', '-contrast', '-enhance', '-quality', '100', str(self.icono_orig)])
        # y por si hubiera fallos en la conversión:
        ret = conv.wait()
        extension = self.icono_orig[-3:]
        if self.icono_orig[-3:] != 'png':
            self.icono_orig = self.icono_orig[:-3] + 'png'
            ficheroborrar = self.icono_orig[:-3]+ extension
            print ficheroborrar
            subprocess.Popen(['rm', ficheroborrar])
        if ret != 0:
            sys.exit(ret)

    def ajusta_escala(self):
        """A un tamaño aproximado de 85x85. Si una de las dos dimensiones 
        es mayor de 90px se reduce el icono, si ambas son menores de 85px 
        hay que aumentarle:"""
        # si alguna de las dos dimensiones es > 90 hay que reducir el icono:
        # leemos self.icono_orig para ver su tamaño:
        im = Image.open(self.icono_orig)
        if im.size[0] > 90 or im.size[1] > 90:
            # máximo posible dentro de una caja de 95x95px: 
            conv = subprocess.Popen(['convert', str(self.icono_orig), '-resample', '300', '-define', 'filter:blur=0.75', '-filter', 'Lanczos', '-thumbnail', '90x90', '-normalize', str(self.icono_orig)])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
        # si ambas son menores de 85px hay que aumentarle:
        if im.size[0] < 85 and im.size[1] < 85:
            conv = subprocess.Popen(['convert', str(self.icono_orig), '-resample', '3000', '-define', 'filter:blur=0.75', '-filter', 'Mitchell', '-adaptive-resize', '82x82', str(self.icono_orig)])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)

    def mejora_contraste(self):
        """Imprescindible y complejo: requiere pruebas con diferentes iconos y
        pueden variar los resultados según vaya evolucionando ImageMagick"""
        conv = subprocess.Popen(['convert', str(self.icono_orig), '-quality', '95', '-modulate', '92,98,100', '-level', '6%,100%,0.9', '-unsharp', '1.5x1.2+1+0.01', str(self.icono_orig)])
        ret = conv.wait()
        if ret != 0:
            sys.exit(ret)

    def mejora_contraste_pil(self):
        """Extraído del módulo PIL, no se utilizará en tamaños pequeños, 
        pero se guarda aquí para posibles usos futuros"""
        from PIL import ImageFilter
        im = Image.open(self.icono_orig)
        im = im.filter( ImageFilter.EDGE_ENHANCE)
        im.save(self.icono_orig)

    def arroja_sombra(self, sombraicono):
        """Sombra específica para el icono, debe indicarse en modo '80x2+2+2' """
        self.sombraicono = sombraicono
        conv = subprocess.Popen(['convert', str(self.icono_orig), '(', '+clone', '-background', 'black', '-shadow', str(self.sombraicono), ')', '+swap', '-background', 'transparent', '-layers', 'merge', '+repage', '-chop', '2x2', str(self.icono_orig)])
        ret = conv.wait()
        if ret != 0:
            print("Fallo en el sombreado, error=%s" % ret)
            sys.exit(ret)
        # y falta autorecortar:
        conv = subprocess.Popen(['convert', str(self.icono_orig), '-trim', str(self.icono_orig)])
        ret = conv.wait()
        if ret != 0:
            print("Fallo en el recortado, error=%s" % ret)
            sys.exit(ret)
        print("...y recortado el sobrante")

    def crea_titulo(self, fuente, colortexto, alturatexto):
        """Para crear el PNG que contiene el título que va en el icono"""
        # por defecto , usar colortexto = "#f7a32e", fuente = DejaVu-Sans-Book, altura = 14
        self.fuente = fuente
        self.colortexto = colortexto
        self.alturatexto = alturatexto
        global numversionIM
        if len(self.titulo) > 60:
            self.titulo = self.titulo[:60]
        # el PNG con el texto tiene 3 posibles dimensiones 
        # dependiendo de la longitud del texto:
        if len(self.titulo) < 15: 
            texto = 'caption' + ':'+self.titulo
            if self.numversionIM > 6.5:
                # para poder usar kerning (desde 6.4.8) y interline-spacing (desde 6.5.5):
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-kerning', '-0.2', '-size', '114x16', '-trim', '+repage', '-pointsize', '14', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/tituloc.png'])
            if self.numversionIM <= 6.5: 
                # entonces posiblemente estemos en una máquina con lenny 
                # (con IM 6.3.7), supondrá en general letras más pequeñas:
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-size', '114x16', '-trim', '+repage', '-pointsize', '14', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/tituloc.png'])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
        if len(self.titulo) >= 15 and len(self.titulo) < 37: 
            # se impone una caja contenedora de h=32px, caption ajusta la
            # altura de texto y los retornos de carro:
            texto = 'caption' + ':'+self.titulo
            if self.numversionIM > 6.5:
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-kerning', '-0.2', '-interline-spacing', '-2', '-size', '114x32', '-trim', '+repage', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/titulol.png'])
            if self.numversionIM <= 6.5:
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-size', '114x32', '-pointsize', '13', '-trim', '+repage', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/titulol.png'])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
        if len(self.titulo) >= 37 and len(self.titulo) < 61: 
            # se impone una caja contenedora de h=40px:
            texto = 'caption' + ':'+self.titulo
            if self.numversionIM > 6.5:
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-kerning', '-0.7', '-interline-spacing', '-4', '-size', '114x40', '-trim', '+repage', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/tituloxl.png'])
            if self.numversionIM <= 6.5:
                conv = subprocess.Popen(['convert', '-background', 'transparent', '-fill', str(self.colortexto), '-font', str(self.fuente), '-size', '114x40', '-pointsize', '12', '-trim', '+repage', '-gravity', 'center', str(texto), '-channel', 'RGBA', '-blur', '0.01x0.01', '/tmp/tituloxl.png'])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
            
    def compone_capas(self, fondo):
        """fondo es el fichero gráfico de fondo de todos los iconos"""
        # leemos self.icono_orig para ver su tamaño:
        im = Image.open(self.icono_orig)
        # y para deducir las coordenadas de posición del icono original:
        h = 64 - im.size[0]/2
        if self.numversionIM > 6.5:
            v = 50 - im.size[1]/2
        if self.numversionIM <= 6.5:
            v = 46 - im.size[1]/2
        #
        # primero yuxtaponemos el icono transformado sobre el fondo:
        posicion = '+' + str(h) + '+' + str(v)
        self.posicion = posicion
        conv = subprocess.Popen(['composite', str(self.icono_orig), '-geometry', str(self.posicion), fondo, str(self.icono_orig)])
        ret = conv.wait()
        if ret != 0:
            sys.exit(ret)
        #
        # y ahora yuxtaponemos sobre él el texto del título:
        if len(self.titulo) < 15: # textos de un sólo renglón
            conv = subprocess.Popen(['composite', '/tmp/tituloc.png', '-gravity', 'South', '-pointsize', '14', '-geometry', '+0+6', str(self.icono_orig), str(self.icono_orig)])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
        if len(self.titulo) >= 15 and len(self.titulo) < 37:
            # supuestamente dos renglones 
            conv = subprocess.Popen(['composite', 'titulol.png', '-gravity', 'South', '-geometry', '+0+6', str(self.icono_orig), str(self.icono_orig)])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
        if len(self.titulo) >= 37 and len(self.titulo) < 61:
            # requiere 3 renglones 
            conv = subprocess.Popen(['composite', 'tituloxl.png', '-gravity', 'South', '-geometry', '+0+5', str(self.icono_orig), str(self.icono_orig)])
            ret = conv.wait()
            if ret != 0:
                sys.exit(ret)
         
    def optimiza(self):
        """ Optimizar almacenamiento del PNG reduciendo su peso"""
        conv = subprocess.Popen(['optipng', '-o6', '-np', str(self.icono_orig)])
        ret = conv.wait()
        if ret != 0:
            sys.exit(ret)
        
        ##
        # HASTA AQUÍ LAS FUNCIONES
        ##

    def main(self):
        """ Lanza la secuencia de instrucciones para rediseñar cada icono"""
        # depende del usuario de trabaja con unas características gráficas u otras:
        usu = subprocess.Popen("whoami",stdout=subprocess.PIPE,shell=True)
        USUARIO = usu.stdout.read()
        if USUARIO == "infantil":
            ICONOFONDO = '/usr/share/pixmaps/linex/fondoinf.png'
            SOMBREADO = '80x2+2+2'
            TITULOFUENTE = 'Quercus-Bold'
            TITULOCOLOR = "#fd0003"
            TITULOALTURA = 17
        elif USUARIO == 'ciclo1y2':
            ICONOFONDO = '/usr/share/pixmaps/linex/fondoc1y2.png'
            SOMBREADO = '80x2+2+2'
            TITULOFUENTE = 'Quercus-Bold'
            TITULOCOLOR = "#268f40"
            TITULOALTURA = 17
        elif USUARIO == 'ciclo3':
            ICONOFONDO = '/usr/share/pixmaps/linex/fondoc3.png'
            SOMBREADO = '80x2+2+2'
            TITULOFUENTE = 'Quercus-Bold'
            TITULOCOLOR = "#653db3"
            TITULOALTURA = 17
        else: 
            ICONOFONDO = '/usr/share/pixmaps/linex/fondo.png'
            SOMBREADO = '80x2+2+2'
            TITULOFUENTE = 'DejaVu-Sans-Book'
            TITULOCOLOR = "#f7a32e"
            TITULOALTURA = 14
        # el icono original se lleva a su directorio de destino antes de transformarlo:
        destino=os.path.expanduser("~/.cache/mis_iconos")
        if os.path.isdir(destino) == True:
            # si ya existe .cache/mis_iconos:  
            shutil.copy(self.icono_orig, destino)
        else:
            # si no existe se crea ~/.cache/mis_iconos:
            subprocess.Popen(['mkdir', destino])
            # y se copia allí el original:
            shutil.copy(self.icono_orig, destino)
        (filepath, filename) = os.path.split(self.icono_orig)
        self.icono_orig = destino +"/"+ filename
        # y se construye el lanzador sobreescribiendo el original:
        self.comprueba_im_opti()
        self.asegura_png()
        self.ajusta_escala()
        self.mejora_contraste()
        self.arroja_sombra(SOMBREADO)
        self.crea_titulo(TITULOFUENTE, TITULOCOLOR, TITULOALTURA)
        self.compone_capas(ICONOFONDO)
        self.optimiza()
 
if  __name__ == '__main__':
    """sólo para pruebas"""
    # icono_orig = 'squeak.png'
    # titulo = 'Squeak'
    # conv_ico = ConversorIcono(icono_orig,titulo)
    # conv_ico.main() 
