#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     malla.py
# Purpose:     Cálculo de coordenada, escalado e información de los iconos
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        17-Feb-2011.
# Author:    Isabel Aparicio Pérez
# Copyright:    2011 - Isabel Aparicio Pérez    <prog5pe@edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################

import ConfigParser
import os
import subprocess
import pdb
from shutil import copy

try:  
    import cPickle as pickle  
except ImportError:  
    import pickle  

CATEGORIAS = 5
LANZADORES = 21 # resultado de 3 filas de 7
CARPETAS = 4 #estas es la variable que indica las carpetas aula, instituto, dpto y personal
LANZADORES_GRANDE=32 #se usara cuando la pantalla sea mas grande de 1500
FIJAMX = 1120 # origen x de la LANZADORES para pantalla 1024x600
FIJAMY = 150 # origen y de la LANZADORES para pantalla 1024x600
FIJAAX = 1120 ## origen x de la linea de iconos de categorias para pantalla 1024x600
FIJAAY = 14 # distancia entre el borde superior de pantalla y borde superior de icono de categoria 
MEDIOA = 75 # distancia horizontal entre iconos de categoria
MEDIOMX = 156 # distancia entre columnas de la LANZADORES 
MEDIOMY = 160 # distancia entre filas de la LANZADORES 
MEDIOGRANDE = 156# esta es la distancia que hay entre el primer icono de la LANZADORES de arriba a la
#derecha y el ultimo icono de la LANZADORES de arriba a la izquierda
FIJAVX=60
FIJAVY=160
MEDIOV=78
CARPETASVACIO=False

Coordenadas=()
Iconos={}
IconosCarpetas={}
IconosCategorias={}
Factor=1

def comprobarVertical(fichero):
    ##realiza una comprobacion si en el fichero tiene la opcion vertical o no para generar despuel 
    ##el calculo de la malla de las categorias en caso de que sea vertical en la margen izda de la
    ##pantalla
    try:
        valor=fichero.getboolean("parametros","Vertical")
    except:
        valor=False
    return valor

def existeFichero(fichero):
    #comprueba si existe el fichero pasado por parametro
    cfg = ConfigParser.ConfigParser()
    if not cfg.read(fichero): #(f)
        print ("El fichero aplicaciones.cfg no se encuentra")
        quit()
    return cfg
    
def listadoCategoriasYLanzadores(cfg):
    #entra el fichero de configuracion aplicaciones.cfg
    #busca en la seccion de categorias
    #el resultado en diccionario es 
    ## {categoria:{aplicacion:[],aplicacion:[],....},
    ##categoria2:{aplicacion:[],aplicacion:[],....}}
    diccionario = {}
    for opt, value in cfg.items('categorias'):
        if opt=='carpetas' and value=='':
            pass
        else:
            lista_iconos=value.split(',')
            nivel1={}
            for icono in lista_iconos:
                resul=subprocess.Popen(args=["which",icono],stdout=subprocess.PIPE)
                if resul != '':
                    nivel1[icono]=[]
            diccionario[opt] = nivel1
    return diccionario 

def listadoLanzadores(cfg):
    ##devuelve una lista con los nombres de los accesos directos que estan en el escritorio
    ##pintados de la forma
    ##[aplicacion,aplicacion1,aplicacion2,aplicacion3,.........]
    global CARPETAS
    lista_iconos=[]
    for opt,value in cfg.items('categorias'):
        if opt=='carpetas':
            CARPETAS=len(value.split(','))
        else:
            for i in value.split(','):
                lista_iconos.append(i)
    return lista_iconos
    
def listadoCategorias(diccionario):
    ##devuelve un diccionario cuya clave es la categoria y cuyo valor es una lista vacia
    ##de la forma {categoria:[],categoria:[],.......}
    categorias = {}
    for i in diccionario.keys():
        if i=='carpetas':
            continue
        else:
            categorias[i]=[]  
    return categorias  

def contenidoListadoCategorias(lista,cfg):
    ##con esta funcion se toma cada uno de los elementos de la lista pasada por parametros
    ##busca ese elemento en el fichero de configuracion pasado como parametro
    ##una vez que lo encuentra almacena todos los datos correspondientes a ese lanzador en la
    ##lista
    
    for i in lista:
        lista[i].append(cfg.get(i,"Type"))
        lista[i].append(cfg.get(i,"Terminal"))
        lista[i].append(cfg.get(i,"Name[es_Es]"))
        lista[i].append(cfg.get(i,"Exec"))
        lista[i].append(cfg.get(i,"Icon"))
        lista[i].append(cfg.get(i,"Iconsii"))

def calculaDimensionActual():
    ##calcula las dimensiones actuales de la pantalla en la que estamos trabajando
    DIMENSIONES = subprocess.Popen("xdpyinfo  | grep -i dimensions: | sed 's/[^0-9]*pixels.*(.*).*//' | sed 's/[^0-9x]*//'",stdout=subprocess.PIPE,shell=True).communicate()[0].strip()
    return DIMENSIONES
    
def calculaDimensionVieja():
    ##comprobamos si existe el fichero xresolucion, si existe leemos la linea que tiene almacenada
    ##que correspondera a la resolucion de la pantalla detectada en el arranque anterior.
    ##si el fichero no existe se crea el fichero y se calcula la dimension actual y se escribe en el fichero
    viejaResolucion = os.path.expanduser("~/.xresolucion")
    #aqui lo que hacemos es si la resolucion que calcula es la misma que la anterior lo que hacemos es salir directamente
    if os.path.exists(viejaResolucion):
        DIMENSION_VIEJA = open(viejaResolucion).readline().strip()
    else:
        f = open (viejaResolucion, "w")
        f.write(calculaDimensionActual())
        DIMENSION_VIEJA = open(viejaResolucion).readline().strip()
    return DIMENSION_VIEJA
   
   
def calcularFactor(DIMENSIONES):
    ##se calcula el factor con el que se van a reescalar los iconos y se va a calcular la malla
    FACTOR=1
    ANCHO = int(DIMENSIONES.split("x")[0])
    ALTO = int(DIMENSIONES.split("x")[1])
    FACTORX = ANCHO / 1280.0
    FACTORY = ALTO / 800.0
    # nos quedamos con el factor mas pequeno para realizar los calculos:
    if FACTORX > FACTORY:
        FACTOR = FACTORY
    elif FACTORX < FACTORY:
        FACTOR = FACTORX
    else:
        FACTOR = 1
    if FACTOR > 1.25:
        FACTOR = 1.25
    return FACTOR

def sacaCoordenadasCategorias(FACTOR,FIJAAX):
    ##retorna una lista de n coordenadas donde n sera el numero que se pase en la constante CATEGORIAS
    ## del tipo (x,y) calculadas arriba a la derecha en la pantalla
    coordenadas_arriba_dcha=[0]*CATEGORIAS
    coorX = int(FIJAAX)
    coorY = int(FIJAAY * FACTOR)
    if FACTOR<1:
        distancia_media = int(MEDIOA * FACTOR)
    else:
        distancia_media = int(MEDIOA)
    i = 0
    for i in range(CATEGORIAS):
        indice = CATEGORIAS - 1 - i
        coordenadas_arriba_dcha[indice] = (coorX,coorY)
        coorX = int(coorX-distancia_media)
    FIJAAX=int(coorX) #es necesario fijar este valor para luego dibujar las carpetas
    print coordenadas_arriba_dcha
    return coordenadas_arriba_dcha
    

def sacaCoordenadasCategoriasVertical(FACTOR):
    ##Se encarga de sacar coordenadas de las categorias cuando el resultado de la consulta 
    ##de la funcion compruebaVertical es verdadero.
    coordenadas_verticales=[0]*CATEGORIAS
    coorX = int(FIJAVX * FACTOR)
    coorY = int(FIJAVY * FACTOR)
    if FACTOR<1:
        distancia_media = int(MEDIOV * FACTOR)    
    else:
		distancia_media=int(MEDIOV)

    for i in range(CATEGORIAS):
        coordenadas_verticales[i]=(coorX,coorY)
        coorY = int(coorY + distancia_media)
    return coordenadas_verticales
        
def sacaCoordenadasCarpetas(FACTOR,MEDIO):
    ##se encarga de  calcular las coordenadas de las carpetas que vamos a situas arriba a la izda
    ##de la pantalla, 
    coorY = int(FIJAAY * FACTOR)
    if FACTOR<1:
        distancia_media =int( MEDIOA * FACTOR    )
    else:
		distancia_media=int(MEDIOA)
    
    coordenadas_arriba_izda=[0]*CARPETAS
    coorX = int(MEDIO)+distancia_media
    
    m=0
    for m in range(CARPETAS):
        indicej = CARPETAS - 1 - m
        coordenadas_arriba_izda[indicej] = (coorX , coorY)
        coorX  = int(coorX  - distancia_media)
    return coordenadas_arriba_izda
    
def calculaFijaMX(DIMENSIONES,FACTOR):
    ##calcula la posicion de arriba a la derecha a partir de la cual se va a empezar a calcular
    ##las posiciones de la malla.
    ##para ello lo que hacemos el calcular el ancho de la pantalla y restarle 180
    ANCHO = int(DIMENSIONES.split("x")[0])
    FIJAMX=ANCHO-(180*FACTOR)
    return FIJAMX
    
def calculaMedioPantalla(DIMENSIONES):
    ##calcula el punto medio de la pantalla
    ANCHO = int(DIMENSIONES.split("x")[0])
    MEDIO = ANCHO / 2
    return MEDIO

def sacaCoordenadasLanzadores(FACTOR,DIMENSIONES,FIJAMX):
    ##calcula la malla de los lanzadores, controla si es una pantalla de mas de 1500 y si es asi
    ##calcula una malla de 32 en vez de una malla de 21 para pintar 4 filas de 8 iconos cada una
    ANCHO = int(DIMENSIONES.split("x")[0])
    if ANCHO > 1500:
        coordenadas_malla=[0]*LANZADORES_GRANDE
        j=7
        k=0
        fila=1
        items_por_fila=8
        maxfilas=5
    else:
        coordenadas_malla=[0]*LANZADORES
        j = 6
        k = 0
        fila = 1
        items_por_fila = 7
        maxfilas=4
    coorX = int(FIJAMX)
    coorY = int(FIJAMY * FACTOR)
    if FACTOR<1:
        distancia_media_X = int(MEDIOMX * FACTOR)
        distancia_media_Y = int(MEDIOMY * FACTOR)
    else: 
        distancia_media_X = int(MEDIOMX)
        distancia_media_Y = int(MEDIOMY)
    while (fila < maxfilas):
        if (fila == maxfilas):
            quit()
        elif k == items_por_fila:
            k = 0
            fila = fila + 1
            coorY = coorY + distancia_media_Y 
            coorX = int(FIJAMX)
            j = (fila * (items_por_fila-1)) + (fila - 1)
        else:
            for l in range(items_por_fila):
                coordenadas_malla[j] = (coorX, coorY)
                k = k + 1
                j = j - 1
                coorX =int(coorX-distancia_media_X)
    return coordenadas_malla

def serializaTodo(archivo,coordenadas_malla, coordenadas_arriba_dcha, coordenadas_arriba_izda):
    f = file(archivo, "w")
    pickle.dump(coordenadas_malla, f)
    pickle.dump(coordenadas_arriba_dcha, f)
    if coordenadas_arriba_izda is not None: pickle.dump(coordenadas_arriba_izda, f)
    f.close()            
    return f
    
def compruebaSiCarpeta(cfg):
    nocarpetas=False
    for opt, value in cfg.items('categorias'):
        if opt=='carpetas' and value=='':
           nocarpetas=True
        else:
           pass
    return nocarpetas
    
def recuperaSerializadosLanzadores(archivo):
    f = file(archivo)
    malla_centro = pickle.load(f)
    arriba_dcha = pickle.load(f)
    try:
        arriba_izda = pickle.load(f)
    except:
        arriba_izda=None
    f.close()
    return (malla_centro,arriba_dcha,arriba_izda)
  
def inicializa(f=None,serializado=None):
    global Coordenadas
    global Iconos
    global Factor
    global Lanzadores
    global CARPETAS
    global CARPETASVACIO
    
    if f is None:
        #first, try to find a user configuration
        f=os.path.expanduser("~/.config/aplicaciones.cfg")
        #if the user has no his own configuration file:
        if not os.path.exists(f): #then use the general configuration
            f='/etc/aplicaciones.cfg'                 
        
    if serializado is None:
        serializado=os.path.expanduser("~/.config/salida_escritorio")
    
    config=existeFichero(f)
    vertical=comprobarVertical(config)
    Iconos=listadoCategoriasYLanzadores(config)
    Iconos["categorias"]=listadoCategorias(Iconos)
    Lanzadores=listadoLanzadores(config)
    CARPETASVACIO=compruebaSiCarpeta(config)

    for cada_categoria in Iconos:
        contenidoListadoCategorias(Iconos[cada_categoria],config)
      
    dim_act=calculaDimensionActual()
    dim_vie=calculaDimensionVieja()
    Factor = calcularFactor(dim_act)
    if dim_act!=dim_vie or not os.path.exists(serializado):        
        ##calculo de las mallas
        if not vertical:
            medio=calculaMedioPantalla(dim_act)
            fijamx=calculaFijaMX(dim_act,Factor)
            md = sacaCoordenadasCategorias(Factor,fijamx)
            mi = sacaCoordenadasCarpetas(Factor,medio)
            mc = sacaCoordenadasLanzadores(Factor,dim_act,fijamx)
            serializaTodo(serializado,mc, md, mi)
        else:
            fijamx=calculaFijaMX(dim_act,Factor)
            md = sacaCoordenadasCategoriasVertical(Factor,fijamx)
            mc = sacaCoordenadasLanzadores(Factor,dim_act,fijamx)
            serializaTodo(serializado,mc,md,None)
        
    if vertical:CARPETAS=0
    Coordenadas = recuperaSerializadosLanzadores(serializado)
          

if  __name__ == '__main__':
    inicializa()
    print Coordenadas
    #print Iconos
    #print Factor

