#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Escritorio Educativo
# Module:     representa_icono.py
# Purpose:     Clase que dibuja y gestiona un icono del escritorio
# Language:    Python 2.5
# Date:        15-Jan-2011.
# Ver.:        17-Feb-2011.
# Author:    Antonio Moreno
# Copyright:    2011 - Antonio Moreno       <antoniomoreno @nospam@ edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################
  
import ConfigParser
import malla

import commands
import comun
import gtk
import cairo
import subprocess
import os.path
import pdb

DIMICOBOTON = [62, 74]
DIMICOLANZADOR = [120, 120]

global objLanzadores
objLanzadores = []
  
class RepresentaIcono():
    """Para representar de uno en uno TODOS los iconos.
    Requiere: coordenadas, nombre del icono, instrucción a ejecutar etc.
    Si es un icono cambiante tiene 'icono1' (desactivado) y 
    'icono2' (activado); 'iconos' es la lista de lanzadores que puede 
    tener asociado -caso de ser un icono de categoría-"""
    def __init__(self, factor, icono1, icono2, icono_a_dibujar, 
                 instruccion, nombre, coordenadas, iconos, activo, tipo):
        """Creamos la ventana transparente para todas las
        instancias, los eventos y los parámetros"""
        self.activo = activo # True si estoy pulsado
        self.tipo = tipo # 1 para categoría y 2 para carpeta
        self.icono1 = icono1
        self.icono2 = icono2
        self.icono_a_dibujar = icono_a_dibujar
        self.instruccion = instruccion
        self.factor = factor
        self.nombre = nombre
        self.coordenadas = coordenadas
        self.iconos = iconos
        #
        # acerca de la VENTANA contenedora:
        import comun
        self.window = comun.DesktopWindow()
        self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
        self.window.set_resizable = True
        self.window.move(self.coordenadas[0],self.coordenadas[1])
        self.window.set_events(gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.BUTTON_PRESS_MASK)
        #
        # acerca de la dimensión de la ventana:
        FACTORICO = self.factor
        if self.factor > 1:
            FACTORICO = 1
        if self.tipo < 3:
            self.w = DIMICOBOTON[0] * FACTORICO
            self.h = int(DIMICOBOTON[1] * FACTORICO)
        else:
            self.w = int(DIMICOLANZADOR[0] * FACTORICO)
            self.h = int(DIMICOLANZADOR[1] * FACTORICO)
        # self.window.set_default_size() = (self.w, self.h)
        self.gtk_screen = self.window.get_screen()
        colormap = self.gtk_screen.get_rgba_colormap()
        if colormap == None:
            colormap = self.gtk_screen.get_rgb_colormap()
        gtk.widget_set_default_colormap(colormap)
        if not self.window.is_composited():
            self.supports_alpha = False 
        else:
            self.supports_alpha = True
        # 
        # acerca de los EVENTOS:
        self.window.connect("expose_event", self.redibuja)
        self.window.connect("enter_notify_event", self.dibuja_veladura)
        self.window.connect("leave_notify_event", self.redibuja)
        # en cuanto al clic, se ejecutará el método 'comando' si es un botón
        # superior y el método 'comando_lanzadores' si es un lanzador:
        if (malla.Iconos['categorias'].keys().count(self.instruccion) == 1): 
            # si tu instrucción es 'educacion' ó 'internet' o...  es que
            # eres un botón de categoría, entonces:  
            self.window.connect("button_press_event", self.comando)        
        else:
            self.window.connect("button_press_event", self.comando_lanzadores) 
   
    def comando(self, widget, event, data=None):
        """Al pulsar sobre un icono de categoria, activado o no.
        Si no está activado, debe activarse, ocultar los lanzadores que 
        pueda haber, mostrar los lanzadores propios y desactivar cualquier 
        otro botón de categoría. Si está activado debe ocultar sus 
        lanzadores y desactivarse """
        if len(objLanzadores) > 0:
            self.ocultar_lanzadores()
        
        self.categoriasOff()  
        if self.activo:
            self.activo = False
        else:
            self.activo = True
            self.mostrar_lanzadores()
        self.repintame()
        
    def categoriasOff(self):
        for icono in malla.IconosCategorias:
            boton=malla.IconosCategorias[icono]
            activo=(boton.icono_a_dibujar==boton.icono1)            
            if activo:
                boton.activo=False
                boton.repintame()

    def repintame(self):
        # si estás activo pinta el icono2...        
        if self.activo:
            icono_a_dibujar = self.icono2
            if self.tipo == 1:
                posi = malla.Iconos['categorias'].keys().index(self.nombre.lower())
                instruc = malla.Iconos['categorias'].values()[posi][3]
            elif self.tipo == 2:
                posi = malla.Iconos['carpetas'].keys().index(self.nombre.lower())
                instruc = malla.Iconos['carpetas'].values()[posi][3]
        else:
            icono_a_dibujar = self.icono1
            posi = malla.Iconos['categorias'].keys().index(self.nombre.lower())
            instruc = malla.Iconos['categorias'].values()[posi][3]    
        categoria = RepresentaIcono(self.factor, self.icono1, self.icono2,
                                    icono_a_dibujar, instruc, self.nombre, 
                                    self.coordenadas, self.iconos, 
                                    self.activo, self.tipo)
        categoria.main()
        categoria.set_keep_below = True

        
    def comando_lanzadores(self, widget, event, data=None):
        """Lo que debe suceder al pulsar sobre un lanzador"""
        conv = subprocess.Popen([self.instruccion], shell=True)
        return False
        gtk.main()
    
    def redibuja(self, widget, event):
        """Redibuja el widget image cada vez que se lanza el evento expose"""
        self.ctx = widget.window.cairo_create()
        # cambiar el cursor al pasar por encima:
            # http://faq.pygtk.org/index.py?req=show&file=faq05.006.htp
            # visibles aquí : http://www.pygtk.org/docs/pygtk/class-gdkcursor.html
        my_cursor = gtk.gdk.Cursor(gtk.gdk.HAND1)
        self.window.window.set_cursor(my_cursor)
        # el redibujado en sí:
        self.dibuja_imagen(self.ctx,0,0,self.icono_a_dibujar)
        self.window.set_keep_below = True

    def dibuja_veladura(self, widget, event, data=None):
        """Este método dibuja la veladura mientras esté el cursor dentro"""
        cr = widget.window.cairo_create()
        cr.save()
        if self.supports_alpha == True:
            cr.set_source_rgba(1, 1, 0.8, 0.2)
        else:
            cr.set_source_rgb(0.5, 0.5, 0.5)
        cr.set_operator(cairo.OPERATOR_ATOP)
        cr.rectangle(0, 0, self.w, self.h)
        cr.fill()
        cr.restore()

    def mostrar_lanzadores(self):
        N = len(malla.Iconos[str(self.nombre.lower())].keys())
        for p in range(N):
            nombre = malla.Iconos[str(self.nombre.lower())].keys()[p]
            icono1 = ''
            icono2 = ''
            ico = malla.Iconos[str(self.nombre.lower())].values()[p][4]
            instruccion = malla.Iconos[str(self.nombre.lower())].values()[p][3]
            coordenadas = malla.Coordenadas[0][p]
            iconos = ''
            tip = 3
            categoriaL = RepresentaIcono(self.factor, icono1, icono2, ico,
                                         instruccion, nombre, coordenadas, 
                                         iconos, self.activo, tip)
            objLanzadores.append(categoriaL)
            categoriaL.main()
            categoriaL.set_keep_below = True

    def ocultar_lanzadores(self):
        """Se ha desactivado la categoría que lo estaba o se ha activado
        otra, en ambos casos hay que ocultar los lanzadores que existan"""
        global objLanzadores
        # representa las categorías "apagadas":
        #self.categoriasOff()
        # y elimina los lanzadores que pueda haber:
        if len(objLanzadores) > 0:
            for i in objLanzadores:
                i.window.destroy()
            objLanzadores = []

    def prepara_lienzo(self):
        """ Usamos Cairo; ctx es el contexto de dibujo de Cairo"""
        self.pixmap = gtk.gdk.Pixmap (None, int(self.w), int(self.h), 1) 
        ctx = self.pixmap.cairo_create()

        if not self.supports_alpha : ctx.set_operator (cairo.OPERATOR_SOURCE)

        self.dibuja_imagen(ctx, 0, 0, self.icono_a_dibujar) 
        if self.window.is_composited(): 
            ctx.rectangle(0, 0, self.w, self.h)
            ctx.fill()
            self.window.input_shape_combine_mask(self.pixmap, 0, 0)
        else:
            self.window.shape_combine_mask(self.pixmap, 0, 0)
        return False

    def dibuja_imagen(self, ctx, x, y, pix):
        """Cairo dibuja el icono especificado con un tamaño determinado"""
        ctx.save()
        ctx.translate(x, y)
        if os.path.exists(pix):
            pixbuf = gtk.gdk.pixbuf_new_from_file(pix) 
            anchoini = pixbuf.get_width()
            altoini = pixbuf.get_height()
            FACTORICO = malla.Factor
            if malla.Factor > 1:
                FACTORICO = 1
            # finalmente hay que escalar el pixbuf (el propio icono):
            pixbuf = pixbuf.scale_simple(int(anchoini * FACTORICO),
                                             int(altoini * FACTORICO),
                                             gtk.gdk.INTERP_BILINEAR)
            # y si se quisiera bajar los bites: 
                # format = cairo.FORMAT_RGB24
            format = cairo.FORMAT_ARGB32
            iw = int(pixbuf.get_width() * FACTORICO)
            ih = int(pixbuf.get_height() * FACTORICO)
            image = cairo.ImageSurface(format, iw, ih) 
            image = ctx.set_source_pixbuf(pixbuf, 0, 0) 
            ctx.paint() 
            pixbuf = None
            image = None
            ctx.restore()
            ctx.clip()
        return ctx

    def mostrar_ventana(self):
        self.window.show_all() # aquí se muestra en pantalla el icono
        while gtk.events_pending():
            gtk.main_iteration()
        self.window.present()
        self.window.grab_focus()
        self.p = 1
        self.window.set_keep_below = True

    def main(self):
        """Para hacer el dibujado inicial del icono:"""
        self.prepara_lienzo() # rellenamos la ventana
        self.mostrar_ventana() # mostramos la ventana

if __name__ == "__main__":
    self.iconos = malla.Iconos
    self.coordenadas = malla.Coordenadas
    gtk.main()
